@@ -51,6 +51,7 @@ import org.apache.ivy.core.resolve.ResolveOptions;
 import org.apache.ivy.core.resolve.ResolvedModuleRevision;
 import org.apache.ivy.core.retrieve.RetrieveEngine;
 import org.apache.ivy.core.retrieve.RetrieveOptions;
+import org.apache.ivy.core.retrieve.RetrieveReport;
 import org.apache.ivy.core.search.ModuleEntry;
 import org.apache.ivy.core.search.OrganisationEntry;
 import org.apache.ivy.core.search.RevisionEntry;
@@ -64,6 +65,7 @@ import org.apache.ivy.plugins.repository.TransferListener;
 import org.apache.ivy.plugins.resolver.BasicResolver;
 import org.apache.ivy.plugins.resolver.DependencyResolver;
 import org.apache.ivy.plugins.trigger.Trigger;
+import org.apache.ivy.util.DateUtil;
 import org.apache.ivy.util.HostUtil;
 import org.apache.ivy.util.Message;
 import org.apache.ivy.util.MessageLoggerEngine;
@@ -129,7 +131,10 @@ public class Ivy {
     
     private static final int KILO = 1024;
 
-    public static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyyyMMddHHmmss");
+    /**
+     * @deprecated Use the {@link DateUtil} utility class instead.
+     */
+    public static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat(DateUtil.DATE_FORMAT_PATTERN);
     
     /**
      * the current version of Ivy, as displayed on the console when 
@@ -543,6 +548,16 @@ public class Ivy {
         }
     }
 
+    public RetrieveReport retrieve(ModuleRevisionId mrid, RetrieveOptions options)
+            throws IOException {
+        pushContext();
+        try {
+            return retrieveEngine.retrieve(mrid, options);
+        } finally {
+            popContext();
+        }
+    }
+
     // ///////////////////////////////////////////////////////////////////////
     // DELIVER
     // ///////////////////////////////////////////////////////////////////////
