@@ -210,7 +210,7 @@ public class ModuleRevisionId extends UnmodifiableExtendableItem {
             // just to get the default branch
             ? (context.peekIvy() == null ? null : context.getSettings().getDefaultBranch(moduleId)) 
             : branch;
-        this.revision = revision == null ? Ivy.getWorkingRevision() : revision;
+        this.revision = revision == null ? Ivy.getWorkingRevision() : normalizeRevision(revision);
         setStandardAttribute(IvyPatternHelper.ORGANISATION_KEY, this.moduleId.getOrganisation());
         setStandardAttribute(IvyPatternHelper.MODULE_KEY, this.moduleId.getName());
         setStandardAttribute(IvyPatternHelper.BRANCH_KEY, this.branch);
@@ -274,7 +274,10 @@ public class ModuleRevisionId extends UnmodifiableExtendableItem {
 
     public String encodeToString() {
         StringBuffer buf = new StringBuffer();
-        Map attributes = getAttributes();
+        Map attributes = new HashMap(getAttributes());
+        attributes.keySet().removeAll(getExtraAttributes().keySet());
+        attributes.putAll(getQualifiedExtraAttributes());
+
         for (Iterator iter = attributes.keySet().iterator(); iter.hasNext();) {
             String attName = (String) iter.next();
             String value = (String) attributes.get(attName);
@@ -334,4 +337,21 @@ public class ModuleRevisionId extends UnmodifiableExtendableItem {
     public String getBranch() {
         return branch;
     }
+    
+    /**
+     * [revision] is a valid revision in maven. This method strips the '[' and ']'
+     * characters. Cfr. http://docs.codehaus.org/x/IGU
+     */
+    private static String normalizeRevision(String revision) {
+        if (revision.startsWith("[") && revision.endsWith("]") && revision.indexOf(',') == -1) {
+            if (IvyPatternHelper.getTokenString(IvyPatternHelper.REVISION_KEY).equals(revision)) {
+                // this is the case when listing dynamic revions
+                return revision;
+            }
+            
+            return revision.substring(1, revision.length() - 1);
+        } else {
+            return revision;
+        }
+    }
 }
