@@ -229,8 +229,9 @@ public class IvyNode implements Comparable {
                         
                         md = module.getDescriptor();
                         confsToFetch.remove("*");
-                        updateConfsToFetch(Arrays.asList(resolveSpecialConfigurations(
-                            getRequiredConfigurations(parent, parentConf), this)));
+                        updateConfsToFetch(Arrays
+                                .asList(resolveSpecialConfigurations(getRequiredConfigurations(
+                                    parent, parentConf))));
                     } else {
                         Message.warn("\tmodule not found: " + getId());
                         resolver.reportFailure();
@@ -287,7 +288,7 @@ public class IvyNode implements Comparable {
         }
     }
 
-    public Collection getDependencies(String rootModuleConf, String[] confs) {
+    public Collection getDependencies(String rootModuleConf, String[] confs, String requestedConf) {
         if (md == null) {
             throw new IllegalStateException(
                     "impossible to get dependencies when data has not been loaded");
@@ -301,18 +302,31 @@ public class IvyNode implements Comparable {
         }
         Collection deps = new HashSet();
         for (int i = 0; i < confs.length; i++) {
-            deps.addAll(getDependencies(rootModuleConf, confs[i], confs[i]));
+            deps.addAll(getDependencies(rootModuleConf, confs[i], requestedConf));
         }
         return deps;
     }
 
-    public Collection getDependencies(String rootModuleConf, String conf, String requestedConf) {
+    /**
+     * Load the dependencies of the current node
+     * <p>
+     * The resulting collection of nodes may have some configuration to load
+     * 
+     * @param rootModuleConf
+     *            the requested configuration of the root module
+     * @param conf
+     *            the configuration to load of this node
+     * @param requestedConf
+     *            the actual node conf requested, possibly extending the <code>conf</code> one.
+     * @return
+     */
+    public Collection/*<IvyNode>*/ getDependencies(String rootModuleConf, String conf, String requestedConf) {
         if (md == null) {
             throw new IllegalStateException(
                     "impossible to get dependencies when data has not been loaded");
         }
         DependencyDescriptor[] dds = md.getDependencies();
-        Collection dependencies = new LinkedHashSet(); // it's important to respect order
+        Map/*<ModuleRevisionId, IvyNode> */ dependencies = new LinkedHashMap(); // it's important to respect order
         for (int i = 0; i < dds.length; i++) {
             DependencyDescriptor dd = data.mediate(dds[i]);
             String[] dependencyConfigurations = dd.getDependencyConfigurations(conf, requestedConf);
@@ -322,15 +336,20 @@ public class IvyNode implements Comparable {
                 continue;
             }
             ModuleRevisionId requestedDependencyRevisionId = dd.getDependencyRevisionId();
-            if (isDependencyModuleExcluded(rootModuleConf, requestedDependencyRevisionId, conf)) {
+            if (isDependencyModuleExcluded(dd, rootModuleConf, requestedDependencyRevisionId, conf)) {
                 // the whole module is excluded, it is considered as not being part of dependencies
                 // at all
                 Message.verbose("excluding " + dd + " in " + conf);
                 continue;
             }
-            IvyNode depNode = data.getNode(
-                requestedDependencyRevisionId);
-            
+
+            // check if not already loaded here
+            IvyNode depNode = (IvyNode) dependencies.get(requestedDependencyRevisionId);
+            if (depNode == null) {
+                // check if not already loaded during the resolve session
+                depNode = data.getNode(requestedDependencyRevisionId);
+            }
+
             if (depNode == null) {
                 depNode = new IvyNode(data, this, dd);
             } else {
@@ -341,17 +360,16 @@ public class IvyNode implements Comparable {
                 }
 
             }
-            String[] confsArray = resolveSpecialConfigurations(dependencyConfigurations,
-                depNode);
+            String[] confsArray = depNode.resolveSpecialConfigurations(dependencyConfigurations);
             Collection confs = Arrays.asList(confsArray);
             depNode.updateConfsToFetch(confs);
             depNode.addRootModuleConfigurations(depNode.usage, rootModuleConf, confsArray);
             depNode.usage.setRequiredConfs(this, conf, confs);
 
             depNode.addCaller(rootModuleConf, this, conf, requestedConf, dependencyConfigurations, dd);
-            dependencies.add(depNode);
+            dependencies.put(requestedDependencyRevisionId, depNode);
         }
-        return dependencies;
+        return dependencies.values();
     }
 
     private void addDependencyDescriptor(IvyNode parent, DependencyDescriptor dd) {
@@ -362,10 +380,40 @@ public class IvyNode implements Comparable {
         return (DependencyDescriptor) dds.get(parent);
     }
 
-    private boolean isDependencyModuleExcluded(String rootModuleConf,
+    private boolean isDependencyModuleExcluded(DependencyDescriptor dd, String rootModuleConf,
             ModuleRevisionId dependencyRevisionId, String conf) {
-        return callers.doesCallersExclude(rootModuleConf, DefaultArtifact.newIvyArtifact(
-            dependencyRevisionId, null));
+        Artifact a = DefaultArtifact.newIvyArtifact(dependencyRevisionId, null);
+        if (isRoot()) {
+            // no callers, but maybe some exclude
+            Boolean exclude = doesExclude(md, rootModuleConf, new String[] {rootModuleConf}, dd, a,
+                new Stack());
+            return exclude == null ? false : exclude.booleanValue();
+        }
+        return callers.doesCallersExclude(rootModuleConf, a);
+    }
+
+    Boolean doesExclude(ModuleDescriptor md, String rootModuleConf, String[] moduleConfs,
+            DependencyDescriptor dd, Artifact artifact, Stack callersStack) {
+        // artifact is excluded if it match any of the exclude pattern for this dependency...
+        if (dd != null) {
+            if (dd.doesExclude(moduleConfs, artifact.getId().getArtifactId())) {
+                return Boolean.TRUE;
+            }
+        }
+        if (md.doesExclude(moduleConfs, artifact.getId().getArtifactId())) {
+            return Boolean.TRUE;
+        }
+        // ... or if it is excluded by all its callers
+        IvyNode c = getData().getNode(md.getModuleRevisionId());
+        if (c != null) {
+            if (callersStack.contains(c.getId())) {
+                // a circular dependency, we cannot be conclusive here
+                return null;
+            }
+            return Boolean.valueOf(c.doesCallersExclude(rootModuleConf, artifact, callersStack));
+        } else {
+            return Boolean.FALSE;
+        }
     }
 
     public boolean hasConfigurationsToLoad() {
@@ -452,18 +500,17 @@ public class IvyNode implements Comparable {
     /**
      * resolve the '*' special configurations if necessary and possible
      */
-    private String[] resolveSpecialConfigurations(String[] dependencyConfigurations, IvyNode node) {
+    private String[] resolveSpecialConfigurations(String[] dependencyConfigurations) {
         if (dependencyConfigurations.length == 1 && dependencyConfigurations[0].startsWith("*")
-                && node != null && node.isLoaded()) {
+                && isLoaded()) {
             String conf = dependencyConfigurations[0];
             if ("*".equals(conf)) {
-                return node.getDescriptor().getPublicConfigurationsNames();
+                return getDescriptor().getPublicConfigurationsNames();
             }
             // there are exclusions in the configuration
             List exclusions = Arrays.asList(conf.substring(2).split("\\!"));
 
-            List ret = new ArrayList(Arrays.asList(node.getDescriptor()
-                    .getPublicConfigurationsNames()));
+            List ret = new ArrayList(Arrays.asList(getDescriptor().getPublicConfigurationsNames()));
             ret.removeAll(exclusions);
 
             return (String[]) ret.toArray(new String[ret.size()]);
@@ -617,14 +664,14 @@ public class IvyNode implements Comparable {
         }
         String defaultConf = getDefaultConf(conf);
         conf = getMainConf(conf);
-        if (md.getConfiguration(conf) == null) {
+        if ((md.getConfiguration(conf) == null) || Configuration.Visibility.PRIVATE.equals(md.getConfiguration(conf).getVisibility())) {
             if ("".equals(defaultConf)) {
                 return new String[0];
             }
             conf = defaultConf;
         }
         if (conf.startsWith("*")) {
-            return resolveSpecialConfigurations(new String[] {conf}, this);
+            return resolveSpecialConfigurations(new String[] {conf});
         } else if (conf.indexOf(',') != -1) {
             String[] confs = conf.split(",");
             for (int i = 0; i < confs.length; i++) {
