@@ -52,6 +52,7 @@ import org.apache.ivy.core.module.descriptor.DefaultDependencyDescriptor;
 import org.apache.ivy.core.module.descriptor.DefaultModuleDescriptor;
 import org.apache.ivy.core.module.descriptor.DependencyDescriptor;
 import org.apache.ivy.core.module.descriptor.ModuleDescriptor;
+import org.apache.ivy.core.module.id.ModuleId;
 import org.apache.ivy.core.module.id.ModuleRevisionId;
 import org.apache.ivy.core.report.ArtifactDownloadReport;
 import org.apache.ivy.core.report.ConfigurationResolveReport;
@@ -65,7 +66,6 @@ import org.apache.ivy.plugins.conflict.ConflictManager;
 import org.apache.ivy.plugins.parser.ModuleDescriptorParser;
 import org.apache.ivy.plugins.parser.ModuleDescriptorParserRegistry;
 import org.apache.ivy.plugins.repository.url.URLResource;
-import org.apache.ivy.plugins.resolver.CacheResolver;
 import org.apache.ivy.plugins.resolver.DependencyResolver;
 import org.apache.ivy.plugins.version.VersionMatcher;
 import org.apache.ivy.util.Message;
@@ -202,9 +202,6 @@ public class ResolveEngine {
             throws ParseException, IOException {
         DependencyResolver oldDictator = getDictatorResolver();
         IvyContext context = IvyContext.getContext();
-        if (options.isUseCacheOnly()) {
-            setDictatorResolver(new CacheResolver(settings));
-        }
         try {
             String[] confs = options.getConfs(md);
             options.setConfs(confs);
@@ -243,13 +240,10 @@ public class ResolveEngine {
 
             // produce resolved ivy file and ivy properties in cache
             ResolutionCacheManager cacheManager = settings.getResolutionCacheManager();
-            File ivyFileInCache = cacheManager.getResolvedIvyFileInCache(md
-                    .getResolvedModuleRevisionId());
-            md.toIvyFile(ivyFileInCache);
+            cacheManager.saveResolvedModuleDescriptor(md);
 
             // we store the resolved dependencies revisions and statuses per asked dependency
-            // revision id,
-            // for direct dependencies only.
+            // revision id, for direct dependencies only.
             // this is used by the deliver task to resolve dynamic revisions to static ones
             File ivyPropertiesInCache = cacheManager.getResolvedIvyPropertiesInCache(
                         md.getResolvedModuleRevisionId());
@@ -262,11 +256,31 @@ public class ResolveEngine {
                         forcedRevisions.put(dependencies[i].getModuleId(), dependencies[i].getResolvedId());
                     }
                 }
-
+                
                 IvyNode root = dependencies[0].getRoot();
+
+                //                <ModuleId,IvyNode>();
+                Map topLevelDeps = new HashMap(); //
                 for (int i = 0; i < dependencies.length; i++) {
                     if (!dependencies[i].hasProblem()) {
                         DependencyDescriptor dd = dependencies[i].getDependencyDescriptor(root);
+                        if (dd != null) {
+                            ModuleId orgMod = dependencies[i].getModuleId();
+                            topLevelDeps.put(orgMod, dependencies[i]);
+                        }
+                    }
+                }
+
+                for (int i = 0; i < dependencies.length; i++) {
+                    if (!dependencies[i].hasProblem() && !dependencies[i].isCompletelyEvicted()) {
+                        DependencyDescriptor dd = dependencies[i].getDependencyDescriptor(root);
+                        if (dd == null) {
+                            ModuleId mid = dependencies[i].getModuleId();
+                            IvyNode tlDep = (IvyNode)topLevelDeps.get(mid);
+                            if (tlDep != null) {
+                                dd = tlDep.getDependencyDescriptor(root);
+                            }
+                        }
                         if (dd != null) {
                             ModuleRevisionId depResolvedId = dependencies[i].getResolvedId();
                             ModuleDescriptor depDescriptor = dependencies[i].getDescriptor();
@@ -298,7 +312,8 @@ public class ResolveEngine {
                             
                             // The evicted modules have no description, so we can't put the status
                             String status = depDescriptor == null ? "?" : depDescriptor.getStatus();
-                            props.put(depRevisionId.encodeToString(), rev + " " + status + " " + forcedRev);
+                            Message.debug("storing dependency " + depResolvedId + " in props");
+                            props.put(depRevisionId.encodeToString(), rev + " " + status + " " + forcedRev + " " + depResolvedId.getBranch());
                         }
                     }
                 }
@@ -306,7 +321,7 @@ public class ResolveEngine {
             FileOutputStream out = new FileOutputStream(ivyPropertiesInCache);
             props.store(out, md.getResolvedModuleRevisionId() + " resolved revisions");
             out.close();
-            Message.verbose("\tresolved ivy file produced in " + ivyFileInCache);
+            Message.verbose("\tresolved ivy file produced in cache");
 
             report.setResolveTime(System.currentTimeMillis() - start);
 
@@ -498,8 +513,11 @@ public class ResolveEngine {
 
     /**
      * Resolve the dependencies of a module without downloading corresponding artifacts. The module
-     * to resolve is given by its module descriptor.This method requires appropriate configuration
+     * to resolve is given by its module descriptor. This method requires appropriate configuration
      * of the ivy instance, especially resolvers.
+     * <p>
+     * The <code>IvyNode</code>s are ordered from the most dependent to the less dependent, so that
+     * an IvyNode is always found in the list after all IvyNode depending directly on it.
      * 
      * @param md
      *            the descriptor of the module for which we want to get dependencies - must not be
@@ -771,7 +789,7 @@ public class ResolveEngine {
 
         // now we can actually resolve this configuration dependencies
         if (!isDependenciesFetched(node.getNode(), conf) && node.isTransitive()) {
-            Collection dependencies = node.getDependencies(conf);
+            Collection/*<VisitNode>*/ dependencies = node.getDependencies(conf);
             for (Iterator iter = dependencies.iterator(); iter.hasNext();) {
                 VisitNode dep = (VisitNode) iter.next();
                 dep.useRealNode(); // the node may have been resolved to another real one while
@@ -1048,16 +1066,20 @@ public class ResolveEngine {
         /*
          * We first try to remove all evicted nodes from the collection of selected nodes to update
          * this collection. If the collection changes, it means that it contained evicted nodes, and
-         * thus is not up to date. In this case we need to compute selected nodes again. Another
-         * case where we need to deeply compute selected nodes is when selectedNodes is empty (not
-         * computed yet) and we aren't in the context of the direct parent of the node.
+         * thus is not up to date.
          */
-        if (selectedNodes.removeAll(toevict) 
-                || (selectedNodes.isEmpty() 
+        boolean evictedInSelected = selectedNodes.removeAll(toevict);
+        /*
+         * Another case where we need to deeply compute selected nodes is when selectedNodes is
+         * empty (not computed yet) and we aren't in the context of the direct parent of the node.
+         */
+        if (evictedInSelected || (selectedNodes.isEmpty() 
                         && !node.getParent().getNode().equals(ancestor.getNode()))) {
+            // In this case we need to compute selected nodes again. 
             Collection deps = ancestor.getNode().getDependencies(
                 node.getRootModuleConf(), 
-                ancestor.getNode().getConfigurations(node.getRootModuleConf()));
+                ancestor.getNode().getConfigurations(node.getRootModuleConf()), 
+                ancestor.getRequestedConf());
             for (Iterator iter = deps.iterator(); iter.hasNext();) {
                 IvyNode dep = (IvyNode) iter.next();
                 if (dep.getModuleId().equals(node.getModuleId())) {
@@ -1072,9 +1094,11 @@ public class ResolveEngine {
              * (otherwise previous block would have been reached). We can compute conflicts based on
              * the parent direct dependencies in current root module conf.
              */
-            Collection parentDepIvyNodes = node.getParent().getNode()
-                        .getDependencies(node.getRootModuleConf(), 
-                            new String[] {node.getParentConf()});
+            VisitNode parent = node.getParent();
+            Collection parentDepIvyNodes = parent.getNode().getDependencies(
+                node.getRootModuleConf(), 
+                parent.getNode().getConfigurations(node.getRootModuleConf()), 
+                parent.getRequestedConf());
             for (Iterator it = parentDepIvyNodes.iterator(); it.hasNext();) {
                 IvyNode parentDep = (IvyNode) it.next();
                 if (parentDep.getModuleId().equals(node.getModuleId())) {
