@@ -154,6 +154,11 @@ public class VisitNode {
         return root;
     }
 
+    /**
+     * Get an ordered collection with the nodes from the root to this node
+     * 
+     * @return
+     */
     public Collection/*<VisitNode>*/ getPath() {
         if (path == null) {
             path = computePath();
@@ -304,9 +309,9 @@ public class VisitNode {
         return loaded;
     }
 
-    public Collection getDependencies(String conf) {
-        Collection deps = node.getDependencies(rootModuleConf, conf, requestedConf);
-        Collection ret = new ArrayList(deps.size());
+    public Collection/*<VisitNode>*/ getDependencies(String conf) {
+        Collection/*<IvyNode>*/ deps = node.getDependencies(rootModuleConf, conf, requestedConf);
+        Collection/*<VisitNode>*/ ret = new ArrayList(deps.size());
         for (Iterator iter = deps.iterator(); iter.hasNext();) {
             IvyNode depNode = (IvyNode) iter.next();
             ret.add(traverseChild(conf, depNode));
