@@ -19,7 +19,6 @@ package org.apache.ivy.core.retrieve;
 
 import java.io.File;
 import java.io.IOException;
-import java.net.URL;
 import java.text.ParseException;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -48,10 +47,7 @@ import org.apache.ivy.core.module.id.ModuleId;
 import org.apache.ivy.core.module.id.ModuleRevisionId;
 import org.apache.ivy.core.report.ArtifactDownloadReport;
 import org.apache.ivy.core.resolve.ResolveOptions;
-import org.apache.ivy.plugins.parser.ModuleDescriptorParser;
-import org.apache.ivy.plugins.parser.ModuleDescriptorParserRegistry;
 import org.apache.ivy.plugins.report.XmlReportParser;
-import org.apache.ivy.plugins.repository.url.URLResource;
 import org.apache.ivy.util.FileUtil;
 import org.apache.ivy.util.Message;
 
@@ -73,9 +69,20 @@ public class RetrieveEngine {
      * localCacheDirectory to determine an ivy report file, used as input for the copy If such a
      * file does not exist for any conf (resolve has not been called before ?) then an
      * IllegalStateException is thrown and nothing is copied.
+     *
+     * @deprecated Use {@link #retrieve(org.apache.ivy.core.module.id.ModuleRevisionId, RetrieveOptions)} instead
      */
-    public int retrieve(ModuleRevisionId mrid, String destFilePattern, RetrieveOptions options)
-            throws IOException {
+    public int retrieve(ModuleRevisionId mrid, String destFilePattern, RetrieveOptions options) throws IOException {
+        RetrieveOptions retieveOptions = new RetrieveOptions(options);
+        retieveOptions.setDestArtifactPattern(destFilePattern);
+
+        RetrieveReport result = retrieve(mrid, retieveOptions);
+        return result.getNbrArtifactsCopied();
+    }
+
+    public RetrieveReport retrieve(ModuleRevisionId mrid, RetrieveOptions options) throws IOException {
+        RetrieveReport report = new RetrieveReport();
+
         ModuleId moduleId = mrid.getModuleId();
         if (LogOptions.LOG_DEFAULT.equals(options.getLog())) {
             Message.info(":: retrieving :: " + moduleId + (options.isSync() ? " [sync]" : ""));
@@ -85,10 +92,8 @@ public class RetrieveEngine {
         Message.verbose("\tcheckUpToDate=" + settings.isCheckUpToDate());
         long start = System.currentTimeMillis();
 
-        destFilePattern = IvyPatternHelper.substituteVariables(destFilePattern, settings
-                .getVariables());
-        String destIvyPattern = IvyPatternHelper.substituteVariables(options.getDestIvyPattern(),
-            settings.getVariables());
+        String destFilePattern = IvyPatternHelper.substituteVariables(options.getDestArtifactPattern(), settings.getVariables());
+        String destIvyPattern = IvyPatternHelper.substituteVariables(options.getDestIvyPattern(), settings.getVariables());
 
         String[] confs = getConfs(mrid, options);
         if (LogOptions.LOG_DEFAULT.equals(options.getLog())) {
@@ -104,7 +109,8 @@ public class RetrieveEngine {
             Map artifactsToCopy = determineArtifactsToCopy(mrid, destFilePattern, options);
             File fileRetrieveRoot = settings.resolveFile(
                 IvyPatternHelper.getTokenRoot(destFilePattern));
-            File ivyRetrieveRoot = destIvyPattern == null 
+            report.setRetrieveRoot(fileRetrieveRoot);
+            File ivyRetrieveRoot = destIvyPattern == null
                 ? null : settings.resolveFile(IvyPatternHelper.getTokenRoot(destIvyPattern));
             Collection targetArtifactsStructure = new HashSet(); // Set(File) set of all paths
             // which should be present at
@@ -113,8 +119,6 @@ public class RetrieveEngine {
             Collection targetIvysStructure = new HashSet(); // same for ivy files
 
             // do retrieve
-            int targetsCopied = 0;
-            int targetsUpToDate = 0;
             long totalCopiedSize = 0;
             for (Iterator iter = artifactsToCopy.keySet().iterator(); iter.hasNext();) {
                 ArtifactDownloadReport artifact = (ArtifactDownloadReport) iter.next();
@@ -144,11 +148,12 @@ public class RetrieveEngine {
                                 new EndRetrieveArtifactEvent(artifact, destFile));
                         }
                         totalCopiedSize += destFile.length();
-                        targetsCopied++;
+                        report.addCopiedFile(destFile, artifact);
                     } else {
                         Message.verbose("\t\tto " + destFile + " [NOT REQUIRED]");
-                        targetsUpToDate++;
+                        report.addUpToDateFile(destFile, artifact);
                     }
+
                     if ("ivy".equals(artifact.getType())) {
                         targetIvysStructure
                                 .addAll(FileUtil.getPathFiles(ivyRetrieveRoot, destFile));
@@ -184,9 +189,9 @@ public class RetrieveEngine {
             }
             long elapsedTime = System.currentTimeMillis() - start;
             String msg = "\t"
-                + targetsCopied
+                + report.getNbrArtifactsCopied()
                 + " artifacts copied"
-                + (settings.isCheckUpToDate() ? (", " + targetsUpToDate + " already retrieved")
+                + (settings.isCheckUpToDate() ? (", " + report.getNbrArtifactsUpToDate() + " already retrieved")
                         : "")
                 + " (" + (totalCopiedSize / KILO) + "kB/" + elapsedTime + "ms)";
             if (LogOptions.LOG_DEFAULT.equals(options.getLog())) {
@@ -197,11 +202,11 @@ public class RetrieveEngine {
             Message.verbose("\tretrieve done (" + (elapsedTime) + "ms)");
             if (this.eventManager != null) {
                 this.eventManager.fireIvyEvent(new EndRetrieveEvent(
-                    mrid, confs, elapsedTime, targetsCopied, targetsUpToDate, totalCopiedSize, 
+                    mrid, confs, elapsedTime, report.getNbrArtifactsCopied(), report.getNbrArtifactsUpToDate(), totalCopiedSize,
                     options));
             }
 
-            return targetsCopied;
+            return report;
         } catch (Exception ex) {
             throw new RuntimeException("problem during retrieve of " + moduleId + ": " + ex, ex);
         }
@@ -211,14 +216,8 @@ public class RetrieveEngine {
         String[] confs = options.getConfs();
         if (confs == null || (confs.length == 1 && "*".equals(confs[0]))) {
             try {
-                File ivyFile = getCache().getResolvedIvyFileInCache(mrid);
-                Message.verbose("no explicit confs given for retrieve, using ivy file: " + ivyFile);
-                URL ivySource = ivyFile.toURI().toURL();
-                URLResource res = new URLResource(ivySource);
-                ModuleDescriptorParser parser = ModuleDescriptorParserRegistry.getInstance()
-                        .getParser(res);
-                Message.debug("using " + parser + " to parse " + ivyFile);
-                ModuleDescriptor md = parser.parseDescriptor(settings, ivySource, false);
+                ModuleDescriptor md = getCache().getResolvedModuleDescriptor(mrid);
+                Message.verbose("no explicit confs given for retrieve, using ivy file: " + md.getResource().getName());
                 confs = md.getConfigurationsNames();
                 options.setConfs(confs);
             } catch (IOException e) {
@@ -298,42 +297,49 @@ public class RetrieveEngine {
                 ArtifactDownloadReport artifact = (ArtifactDownloadReport) iter.next();
                 String destPattern = "ivy".equals(artifact.getType()) ? destIvyPattern
                         : destFilePattern;
-
+                
                 if (!"ivy".equals(artifact.getType())
                         && !options.getArtifactFilter().accept(artifact.getArtifact())) {
                     continue; // skip this artifact, the filter didn't accept it!
                 }
 
-                String destFileName = IvyPatternHelper.substitute(destPattern, 
-                        artifact.getArtifact().getModuleRevisionId(), artifact.getArtifact(), 
-                        conf, artifact.getArtifactOrigin());
-
+                String destFileName = IvyPatternHelper.substitute(destPattern,
+                            artifact.getArtifact().getModuleRevisionId(), artifact.getArtifact(),
+                            conf, artifact.getArtifactOrigin());
                 Set dest = (Set) artifactsToCopy.get(artifact);
                 if (dest == null) {
                     dest = new HashSet();
                     artifactsToCopy.put(artifact, dest);
                 }
                 String copyDest = settings.resolveFile(destFileName).getAbsolutePath();
-                dest.add(copyDest);
 
-                Set conflicts = (Set) conflictsMap.get(copyDest);
-                Set conflictsReports = (Set) conflictsReportsMap.get(copyDest);
-                Set conflictsConf = (Set) conflictsConfMap.get(copyDest);
-                if (conflicts == null) {
-                    conflicts = new HashSet();
-                    conflictsMap.put(copyDest, conflicts);
+                String[] destinations = new String[] {copyDest};
+                if (options.getMapper() != null) {
+                    destinations = options.getMapper().mapFileName(copyDest);
                 }
-                if (conflictsReports == null) {
-                    conflictsReports = new HashSet();
-                    conflictsReportsMap.put(copyDest, conflictsReports);
-                }
-                if (conflictsConf == null) {
-                    conflictsConf = new HashSet();
-                    conflictsConfMap.put(copyDest, conflictsConf);
-                }
-                if (conflicts.add(artifact.getArtifact().getId())) {
-                    conflictsReports.add(artifact);
-                    conflictsConf.add(conf);
+
+                for (int j = 0; j < destinations.length; j++) {
+                    dest.add(destinations[j]);
+
+                    Set conflicts = (Set) conflictsMap.get(destinations[j]);
+                    Set conflictsReports = (Set) conflictsReportsMap.get(destinations[j]);
+                    Set conflictsConf = (Set) conflictsConfMap.get(destinations[j]);
+                    if (conflicts == null) {
+                        conflicts = new HashSet();
+                        conflictsMap.put(destinations[j], conflicts);
+                    }
+                    if (conflictsReports == null) {
+                        conflictsReports = new HashSet();
+                        conflictsReportsMap.put(destinations[j], conflictsReports);
+                    }
+                    if (conflictsConf == null) {
+                        conflictsConf = new HashSet();
+                        conflictsConfMap.put(destinations[j], conflictsConf);
+                    }
+                    if (conflicts.add(artifact.getArtifact().getId())) {
+                        conflictsReports.add(artifact);
+                        conflictsConf.add(conf);
+                    }
                 }
             }
         }
