@@ -31,7 +31,7 @@ public class RetrieveOptions extends LogOptions {
     public static final String OVERWRITEMODE_ALWAYS = "always";
     public static final String OVERWRITEMODE_NEWER = "newer";
     public static final String OVERWRITEMODE_DIFFERENT = "different";
-    
+
     /**
      * The names of configurations to retrieve. If the array consists only of '*', then all
      * configurations of the module will be retrieved.
@@ -44,6 +44,11 @@ public class RetrieveOptions extends LogOptions {
      */
     private String destIvyPattern = null;
 
+    /**
+     * The pattern to which artifacts should be retrieved.
+     */
+    private String destArtifactPattern = null;
+
     /**
      * The filter to apply before retrieving artifacts.
      */
@@ -73,6 +78,34 @@ public class RetrieveOptions extends LogOptions {
      * The id used to store the resolve information.
      */
     private String resolveId;
+    
+    private FileNameMapper mapper;
+
+    public RetrieveOptions() {
+    }
+
+    public RetrieveOptions(RetrieveOptions options) {
+        super(options);
+        this.confs = options.confs;
+        this.destIvyPattern = options.destIvyPattern;
+        this.destArtifactPattern = options.destArtifactPattern;
+        this.artifactFilter = options.artifactFilter;
+        this.sync = options.sync;
+        this.overwriteMode = options.overwriteMode;
+        this.useOrigin = options.useOrigin;
+        this.makeSymlinks = options.makeSymlinks;
+        this.resolveId = options.resolveId;
+        this.mapper = options.mapper;
+    }
+
+    public String getDestArtifactPattern() {
+        return destArtifactPattern;
+    }
+
+    public RetrieveOptions setDestArtifactPattern(String destArtifactPattern) {
+        this.destArtifactPattern = destArtifactPattern;
+        return this;
+    }
 
     public Filter getArtifactFilter() {
         return artifactFilter;
@@ -146,4 +179,13 @@ public class RetrieveOptions extends LogOptions {
         return this;
     }
 
+    public FileNameMapper getMapper() {
+        return mapper;
+    }
+
+    public RetrieveOptions setMapper(FileNameMapper mapper) {
+        this.mapper = mapper;
+        return this;
+    }
+    
 }
