@@ -59,6 +59,7 @@ import org.apache.ivy.core.resolve.ResolveEngineSettings;
 import org.apache.ivy.core.resolve.ResolveOptions;
 import org.apache.ivy.core.retrieve.RetrieveEngineSettings;
 import org.apache.ivy.core.sort.SortEngineSettings;
+import org.apache.ivy.osgi.core.OsgiLatestStrategy;
 import org.apache.ivy.plugins.IvySettingsAware;
 import org.apache.ivy.plugins.circular.CircularDependencyStrategy;
 import org.apache.ivy.plugins.circular.ErrorCircularDependencyStrategy;
@@ -92,6 +93,7 @@ import org.apache.ivy.plugins.resolver.ChainResolver;
 import org.apache.ivy.plugins.resolver.DependencyResolver;
 import org.apache.ivy.plugins.resolver.DualResolver;
 import org.apache.ivy.plugins.resolver.ResolverSettings;
+import org.apache.ivy.plugins.signer.SignatureGenerator;
 import org.apache.ivy.plugins.trigger.Trigger;
 import org.apache.ivy.plugins.version.ChainVersionMatcher;
 import org.apache.ivy.plugins.version.ExactVersionMatcher;
@@ -155,6 +157,9 @@ public class IvySettings implements SortEngineSettings, PublishEngineSettings, P
 
     // Map (String name -> RepositoryCacheManager)
     private Map repositoryCacheManagers = new HashMap(); 
+    
+    // Map (String name -> SignatureGenerator)
+    private Map signatureGenerators = new HashMap();
 
     // List (Trigger)
     private List triggers = new ArrayList(); 
@@ -243,11 +248,13 @@ public class IvySettings implements SortEngineSettings, PublishEngineSettings, P
         LatestLexicographicStrategy latestLexicographicStrategy = new LatestLexicographicStrategy();
         LatestRevisionStrategy latestRevisionStrategy = new LatestRevisionStrategy();
         LatestTimeStrategy latestTimeStrategy = new LatestTimeStrategy();
+        OsgiLatestStrategy osgiLatestStrategy = new OsgiLatestStrategy();
 
         addLatestStrategy("latest-revision", latestRevisionStrategy);
         addLatestStrategy("latest-lexico", latestLexicographicStrategy);
         addLatestStrategy("latest-time", latestTimeStrategy);
-        
+        addLatestStrategy("latest-osgi", osgiLatestStrategy);
+
         addLockStrategy("no-lock", new NoLockStrategy());
         addLockStrategy("artifact-lock", new ArtifactLockStrategy(debugLocking()));
 
@@ -293,7 +300,7 @@ public class IvySettings implements SortEngineSettings, PublishEngineSettings, P
 
     private void addSystemProperties() {
         try {
-            addAllVariables(System.getProperties());
+            addAllVariables((Map) System.getProperties().clone());
         } catch (AccessControlException ex) {
             Message.verbose(
                 "access denied to getting all system properties: they won't be available as Ivy variables."
@@ -684,6 +691,19 @@ public class IvySettings implements SortEngineSettings, PublishEngineSettings, P
     public void addConfigured(ModuleDescriptorParser parser) {
         ModuleDescriptorParserRegistry.getInstance().addParser(parser);
     }
+    
+    public void addConfigured(SignatureGenerator generator) {
+        addSignatureGenerator(generator);
+    }
+    
+    public void addSignatureGenerator(SignatureGenerator generator) {
+        init(generator);
+        signatureGenerators.put(generator.getName(), generator);
+    }
+    
+    public SignatureGenerator getSignatureGenerator(String name) {
+        return (SignatureGenerator) signatureGenerators.get(name);
+    }
 
     public void addResolver(DependencyResolver resolver) {
         if (resolver == null) {
@@ -723,6 +743,9 @@ public class IvySettings implements SortEngineSettings, PublishEngineSettings, P
 
     public void setDefaultResolver(String resolverName) {
         checkResolverName(resolverName);
+        if (resolverName != null && !resolverName.equals(defaultResolverName)) {
+            defaultResolver = null;
+        }
         defaultResolverName = resolverName;
     }
 
@@ -852,6 +875,10 @@ public class IvySettings implements SortEngineSettings, PublishEngineSettings, P
         return getResolver(resolverName);
     }
 
+    public boolean hasResolver(String resolverName) {
+        return resolversMap.containsKey(resolverName);
+    }
+
     public DependencyResolver getResolver(String resolverName) {
         if (dictatorResolver != null) {
             return dictatorResolver;
@@ -1225,6 +1252,7 @@ public class IvySettings implements SortEngineSettings, PublishEngineSettings, P
         if (resolutionCacheManager == null) {
             resolutionCacheManager 
                 = new DefaultResolutionCacheManager(getDefaultResolutionCacheBasedir());
+            init(resolutionCacheManager);
         }
         return resolutionCacheManager;
     }
@@ -1457,5 +1485,8 @@ public class IvySettings implements SortEngineSettings, PublishEngineSettings, P
         }
     }
 
-    
+    public Namespace getContextNamespace() {
+        return Namespace.SYSTEM_NAMESPACE;
+    }
+
 }
