@@ -30,17 +30,23 @@ import java.util.Map;
 import org.apache.ivy.core.IvyContext;
 import org.apache.ivy.core.cache.ArtifactOrigin;
 import org.apache.ivy.core.module.descriptor.Artifact;
+import org.apache.ivy.core.module.descriptor.Configuration;
 import org.apache.ivy.core.module.descriptor.DefaultArtifact;
 import org.apache.ivy.core.module.descriptor.DefaultDependencyDescriptor;
 import org.apache.ivy.core.module.descriptor.DependencyDescriptor;
 import org.apache.ivy.core.module.descriptor.ModuleDescriptor;
+import org.apache.ivy.core.module.descriptor.Configuration.Visibility;
 import org.apache.ivy.core.module.id.ModuleRevisionId;
 import org.apache.ivy.core.resolve.ResolveData;
 import org.apache.ivy.core.resolve.ResolveEngine;
 import org.apache.ivy.core.resolve.ResolveOptions;
 import org.apache.ivy.core.resolve.ResolvedModuleRevision;
+import org.apache.ivy.plugins.namespace.NameSpaceHelper;
 import org.apache.ivy.plugins.parser.ModuleDescriptorParser;
 import org.apache.ivy.plugins.parser.ParserSettings;
+import org.apache.ivy.plugins.parser.m2.PomModuleDescriptorBuilder.PomDependencyDescriptor;
+import org.apache.ivy.plugins.parser.m2.PomReader.PomDependencyData;
+import org.apache.ivy.plugins.parser.m2.PomReader.PomDependencyMgtElement;
 import org.apache.ivy.plugins.parser.xml.XmlModuleDescriptorWriter;
 import org.apache.ivy.plugins.repository.Resource;
 import org.apache.ivy.plugins.repository.url.URLResource;
@@ -114,7 +120,9 @@ public final class PomModuleDescriptorParser implements ModuleDescriptorParser {
         try {           
             PomReader domReader = new PomReader(descriptorURL, res);            
             domReader.setProperty("parent.version", domReader.getParentVersion());
+            domReader.setProperty("parent.groupId", domReader.getParentGroupId());
             domReader.setProperty("project.parent.version", domReader.getParentVersion());
+            domReader.setProperty("project.parent.groupId", domReader.getParentGroupId());
 
             Map pomProperties = domReader.getPomProperties();
             for (Iterator iter = pomProperties.entrySet().iterator(); iter.hasNext();) {
@@ -136,7 +144,7 @@ public final class PomModuleDescriptorParser implements ModuleDescriptorParser {
                 if (parentModule != null) {
                     parentDescr = parentModule.getDescriptor();
                 } else {
-                    throw new IOException("Impossible to load parent for " + descriptorURL + "."
+                    throw new IOException("Impossible to load parent for " + res.getName() + "."
                        + " Parent=" + parentModRevID);
                 }
                 if (parentDescr != null) {
@@ -189,12 +197,12 @@ public final class PomModuleDescriptorParser implements ModuleDescriptorParser {
                     DefaultDependencyDescriptor dd = new DefaultDependencyDescriptor(mdBuilder
                             .getModuleDescriptor(), relocation, true, false, true);
                     /* Map all public dependencies */
-                    dd.addDependencyConfiguration("compile", "compile");
-                    dd.addDependencyConfiguration("runtime", "runtime");
-                    dd.addDependencyConfiguration("default", "default");
-                    dd.addDependencyConfiguration("master", "master");
-                    dd.addDependencyConfiguration("provided", "provided");
-                    dd.addDependencyConfiguration("system", "system");
+                    Configuration[] m2Confs = PomModuleDescriptorBuilder.MAVEN2_CONFIGURATIONS;
+                    for (int i = 0; i < m2Confs.length; i++) {
+                        if (Visibility.PUBLIC.equals(m2Confs[i].getVisibility())) {
+                            dd.addDependencyConfiguration(m2Confs[i].getName(), m2Confs[i].getName());
+                        }
+                    }
                     mdBuilder.addDependency(dd);
                 }
             } else {                            
@@ -214,7 +222,11 @@ public final class PomModuleDescriptorParser implements ModuleDescriptorParser {
                     // add dependency management info from parent
                     List depMgt = PomModuleDescriptorBuilder.getDependencyManagements(parentDescr);
                     for (Iterator it = depMgt.iterator(); it.hasNext();) {
-                        mdBuilder.addDependencyMgt((PomDependencyMgt) it.next());
+                        PomDependencyMgt dep = (PomDependencyMgt) it.next();
+                        if (dep instanceof PomDependencyMgtElement) {
+                            dep = domReader.new PomDependencyMgtElement((PomDependencyMgtElement) dep);
+                        }
+                        mdBuilder.addDependencyMgt(dep);
                     }
                     
                     // add plugins from parent
@@ -227,7 +239,35 @@ public final class PomModuleDescriptorParser implements ModuleDescriptorParser {
                 
                 for (Iterator it = domReader.getDependencyMgt().iterator(); it.hasNext();) {
                     PomDependencyMgt dep = (PomDependencyMgt) it.next();
-                    mdBuilder.addDependencyMgt(dep);
+                    if ("import".equals(dep.getScope())) {
+                        ModuleRevisionId importModRevID = ModuleRevisionId.newInstance(
+                            dep.getGroupId(), 
+                            dep.getArtifactId(), 
+                            dep.getVersion());
+                        ResolvedModuleRevision importModule = parseOtherPom(ivySettings, 
+                            importModRevID);
+                        if (importModule != null) {
+                            ModuleDescriptor importDescr = importModule.getDescriptor();
+                            
+                            // add dependency management info from imported module
+                            List depMgt = PomModuleDescriptorBuilder.getDependencyManagements(importDescr);
+                            for (Iterator it2 = depMgt.iterator(); it2.hasNext();) {
+                                PomDependencyMgt importedDepMgt = (PomDependencyMgt) it2.next();
+                                mdBuilder.addDependencyMgt(new DefaultPomDependencyMgt(
+                                        importedDepMgt.getGroupId(),
+                                        importedDepMgt.getArtifactId(),
+                                        importedDepMgt.getVersion(),
+                                        importedDepMgt.getScope(), 
+                                        importedDepMgt.getExcludedModules()));
+                            }
+                        } else {
+                            throw new IOException("Impossible to import module for " + res.getName() + "."
+                               + " Import=" + importModRevID);
+                        }
+                        
+                    } else {
+                        mdBuilder.addDependencyMgt(dep);
+                    }
                 }
                 
                 for (Iterator it = domReader.getDependencies().iterator(); it.hasNext();) {
@@ -237,7 +277,14 @@ public final class PomModuleDescriptorParser implements ModuleDescriptorParser {
 
                 if (parentDescr != null) {
                     for (int i = 0; i < parentDescr.getDependencies().length; i++) {
-                        mdBuilder.addDependency(parentDescr.getDependencies()[i]);
+                        DependencyDescriptor descriptor = parentDescr.getDependencies()[i];
+                        if (descriptor instanceof PomDependencyDescriptor) {
+                            PomDependencyData parentDep = ((PomDependencyDescriptor) descriptor).getPomDependencyData();
+                            PomDependencyData dep = domReader.new PomDependencyData(parentDep);
+                            mdBuilder.addDependency(res, dep);
+                        } else {
+                            mdBuilder.addDependency(descriptor);
+                        }
                     }
                 }
                 
@@ -283,7 +330,16 @@ public final class PomModuleDescriptorParser implements ModuleDescriptorParser {
                     Message.debug("source artifact found for " + mrid);
                     mdBuilder.addSourceArtifact();
                 } else {
-                    Message.debug("no source artifact found for " + mrid);
+                    // it seems that sometimes the 'src' classifier is used instead of 'sources'
+                    // Cfr. IVY-1138
+                    ArtifactOrigin srcArtifact = resolver.locate(mdBuilder.getSrcArtifact());
+                    if (!ArtifactOrigin.isUnknown(srcArtifact)
+                            && !srcArtifact.getLocation().equals(mainArtifactLocation)) {
+                        Message.debug("source artifact found for " + mrid);
+                        mdBuilder.addSrcArtifact();
+                    } else {
+                        Message.debug("no source artifact found for " + mrid);
+                    }
                 }
                 ArtifactOrigin javadocArtifact = resolver.locate(mdBuilder.getJavadocArtifact());
                 if (!ArtifactOrigin.isUnknown(javadocArtifact) 
@@ -313,6 +369,7 @@ public final class PomModuleDescriptorParser implements ModuleDescriptorParser {
             // TODO: Throw exception here?
             return null;
         } else {
+            dd = NameSpaceHelper.toSystem(dd, ivySettings.getContextNamespace());
             ResolvedModuleRevision otherModule = resolver.getDependency(dd, data);
             return otherModule;
         }
