@@ -21,18 +21,20 @@ import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.net.MalformedURLException;
+import java.net.URI;
+import java.net.URISyntaxException;
 import java.net.URL;
 import java.text.ParseException;
 import java.util.Arrays;
 import java.util.LinkedHashMap;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
 
 import javax.xml.parsers.ParserConfigurationException;
 
 import org.apache.ivy.Ivy;
 import org.apache.ivy.core.IvyContext;
-import org.apache.ivy.core.cache.ResolutionCacheManager;
 import org.apache.ivy.core.module.descriptor.Configuration;
 import org.apache.ivy.core.module.descriptor.ConfigurationAware;
 import org.apache.ivy.core.module.descriptor.DefaultArtifact;
@@ -67,8 +69,11 @@ import org.apache.ivy.plugins.parser.ModuleDescriptorParser;
 import org.apache.ivy.plugins.parser.ModuleDescriptorParserRegistry;
 import org.apache.ivy.plugins.parser.ParserSettings;
 import org.apache.ivy.plugins.repository.Resource;
+import org.apache.ivy.plugins.repository.file.FileResource;
 import org.apache.ivy.plugins.repository.url.URLResource;
 import org.apache.ivy.plugins.resolver.DependencyResolver;
+import org.apache.ivy.util.FileUtil;
+import org.apache.ivy.util.DateUtil;
 import org.apache.ivy.util.Message;
 import org.apache.ivy.util.XMLHelper;
 import org.apache.ivy.util.extendable.ExtendableItemHelper;
@@ -82,7 +87,7 @@ import org.xml.sax.SAXException;
 public class XmlModuleDescriptorParser extends AbstractModuleDescriptorParser {
     static final String[] DEPENDENCY_REGULAR_ATTRIBUTES = new String[] {"org", "name", "branch",
             "branchConstraint", "rev", "revConstraint", "force", "transitive", "changing", "conf"};
-
+    
     private static final XmlModuleDescriptorParser INSTANCE = new XmlModuleDescriptorParser();
 
     public static XmlModuleDescriptorParser getInstance() {
@@ -202,7 +207,7 @@ public class XmlModuleDescriptorParser extends AbstractModuleDescriptorParser {
         }
 
         protected static final List ALLOWED_VERSIONS = Arrays.asList(
-            new String[] {"1.0", "1.1", "1.2", "1.3", "1.4", "2.0", "2.1", "2.2"});
+            new String[] {"1.0", "1.1", "1.2", "1.3", "1.4", "2.0", "2.1", "2.2", "2.3"});
 
         /* how and what do we have to parse */
         private ParserSettings settings;
@@ -362,99 +367,95 @@ public class XmlModuleDescriptorParser extends AbstractModuleDescriptorParser {
             }
         }
 
+        /**
+         * Default parent location to check (for dev ONLY) 
+         * @return a relative path to a parent module descriptor
+         */
         protected String getDefaultParentLocation() {
             return "../ivy.xml";
         }
 
+        /**
+         * Handle extends elements.
+         * It checks :
+         *  <ul>
+         *      <li>filesystem based on location attribute, if no one is specified it will check the default parent location</li>
+         *      <li>cache to find a resolved parent descriptor</li>
+         *      <li>ask repositories to retrieve the parent module descriptor</li>
+         *  </ul>
+         * @param attributes 
+         * @throws ParseException 
+         */
         protected void extendsStarted(Attributes attributes) throws ParseException {
-            String parentOrganisation = attributes.getValue("organisation");
-            String parentModule = attributes.getValue("module");
-            String parentRevision = attributes.getValue("revision");
-            String location = attributes.getValue("location") != null ? attributes
-                    .getValue("location") : getDefaultParentLocation();
+            String parentOrganisation = settings.substitute(attributes.getValue("organisation"));
+            String parentModule = settings.substitute(attributes.getValue("module"));
+            String parentRevision = attributes.getValue("revision") != null ? settings.substitute(attributes
+                    .getValue("revision")) : Ivy.getWorkingRevision();
+            String location = attributes.getValue("location") != null ? settings.substitute(attributes
+                    .getValue("location")) : getDefaultParentLocation();
             ModuleDescriptor parent = null;
 
-            String extendType = attributes.getValue("extendType") != null ? attributes.getValue(
-                "extendType").toLowerCase() : "all";
+            String extendType = attributes.getValue("extendType") != null ? settings.substitute(attributes.getValue(
+                "extendType").toLowerCase(Locale.US)) : "all";
 
             List/* <String> */extendTypes = Arrays.asList(extendType.split(","));
+            ModuleId parentMid = new ModuleId(parentOrganisation, parentModule);
+            ModuleRevisionId parentMrid = new ModuleRevisionId(parentMid, parentRevision);
 
+            //check on filesystem based on location attribute (for dev ONLY)
+            boolean local = false;
             try {
-                Message.debug("Trying to parse included ivy file :" + location);
-                parent = parseOtherIvyFileOnFileSystem(location);
-
-                //verify that the parsed descriptor is the correct parent module.
-                ModuleId expected = new ModuleId(parentOrganisation, parentModule);
-                ModuleId pid = parent.getModuleRevisionId().getModuleId();
-                if (!expected.equals(pid)) {
-                    Message.verbose("Ignoring parent Ivy file " + location + "; expected "
-                        + expected + " but found " + pid);
-                    parent = null;
+                parent = parseParentModuleOnFilesystem(location);
+                if (parent != null) {
+                    ModuleId foundMid = parent.getResolvedModuleRevisionId().getModuleId();
+                    if (!foundMid.equals(parentMid)) {
+                        // the filesystem contains a parent module with different organisation
+                        // or module name; ignore that parent module
+                        Message.info("Found a parent module with unexpected ModuleRevisionId at source location " 
+                                + location + "! Expected: " + parentMid + ". Found: " + foundMid 
+                                + ". This parent module will be ignored.");
+                        parent = null;
+                    }
                 }
                 
-            } catch (ParseException e) {
-                Message.warn("Unable to parse included ivy file " + location + ": " 
-                    + e.getMessage());
+                local = parent != null;
             } catch (IOException e) {
                 Message.warn("Unable to parse included ivy file " + location + ": " 
                     + e.getMessage());
             }
-
-            // if the included ivy file is not found on file system, tries to resolve using
-            // repositories
+            
+            // if not found, tries to resolve using repositories
             if (parent == null) {
                 try {
-                    Message.debug(
-                        "Trying to parse included ivy file by asking repository for module :"
-                                    + parentOrganisation
-                                    + "#"
-                                    + parentModule
-                                    + ";"
-                                    + parentRevision);
-                    parent = parseOtherIvyFile(parentOrganisation, parentModule, parentRevision);
+                    parent = parseOtherIvyFile(parentMrid);
                 } catch (ParseException e) {
-                    Message.warn("Unable to parse included ivy file for " + parentOrganisation
-                            + "#" + parentModule + ";" + parentRevision);
+                    Message.warn("Unable to parse included ivy file for " + parentMrid.toString());
                 }
             }
-
+            
+            // if still not found throw an exception
             if (parent == null) {
                 throw new ParseException("Unable to parse included ivy file for "
-                        + parentOrganisation + "#" + parentModule + ";" + parentRevision, 0);
-            }
-
-            ResolutionCacheManager cacheManager = settings.getResolutionCacheManager();
-
-            File ivyFileInCache = cacheManager.getResolvedIvyFileInCache(parent
-                    .getResolvedModuleRevisionId());
-            //Generate the parent cache file if necessary
-            if (parent.getResource() != null
-                    && !parent.getResource().getName().equals(ivyFileInCache.toURI().toString())) {
-                try {
-                    parent.toIvyFile(ivyFileInCache);
-                } catch (ParseException e) {
-                    throw new ParseException("Unable to create cache file for "
-                            + parentOrganisation + "#" + parentModule + ";" + parentRevision
-                            + " Reason:" + e.getLocalizedMessage(), 0);
-                } catch (IOException e) {
-                    throw new ParseException("Unable to create cache file for "
-                            + parentOrganisation + "#" + parentModule + ";" + parentRevision
-                            + " Reason :" + e.getLocalizedMessage(), 0);
-                }
+                        + parentMrid.toString(), 0);
             }
 
             DefaultExtendsDescriptor ed = new DefaultExtendsDescriptor(
-                    parent.getModuleRevisionId(),
-                    parent.getResolvedModuleRevisionId(),
-                    attributes.getValue("location"),
-                    (String[]) extendTypes.toArray(new String[extendTypes.size()]));
+                    parent,
+                    location,
+                    (String[]) extendTypes.toArray(new String[extendTypes.size()]),
+                    local);
             getMd().addInheritedDescriptor(ed);
 
             mergeWithOtherModuleDescriptor(extendTypes, parent);
         }
 
+        /**
+         * Merge current module with a given module descriptor and specify what should be inherited through extendTypes argument
+         * @param extendTypes specify what should be inherited
+         * @param parent a given parent module descriptor
+         */
         protected void mergeWithOtherModuleDescriptor(List/* <String> */extendTypes,
-                ModuleDescriptor parent) {
+                ModuleDescriptor parent) throws ParseException {
 
             if (extendTypes.contains("all")) {
                 mergeAll(parent);
@@ -464,7 +465,7 @@ public class XmlModuleDescriptorParser extends AbstractModuleDescriptorParser {
                 }
                 
                 if (extendTypes.contains("configurations")) {
-                    mergeConfigurations(parent.getModuleRevisionId(), parent.getConfigurations());
+                    mergeConfigurations(parent);
                 }
 
                 if (extendTypes.contains("dependencies")) {
@@ -474,18 +475,29 @@ public class XmlModuleDescriptorParser extends AbstractModuleDescriptorParser {
                 if (extendTypes.contains("description")) {
                     mergeDescription(parent.getDescription());
                 }
+                if (extendTypes.contains("licenses")) {
+                    mergeLicenses(parent.getLicenses());
+                }
             }
 
         }
 
+        /**
+         * Merge everything from a given parent
+         * @param parent a given parent module desciptor
+         */
         protected void mergeAll(ModuleDescriptor parent) {
-            ModuleRevisionId sourceMrid = parent.getModuleRevisionId();
             mergeInfo(parent);
-            mergeConfigurations(sourceMrid, parent.getConfigurations());
+            mergeConfigurations(parent);
             mergeDependencies(parent.getDependencies());
             mergeDescription(parent.getDescription());
+            mergeLicenses(parent.getLicenses());
         }
-        
+
+        /**
+         * Explain how to inherit metadatas related to info element
+         * @param parent a given parent module decriptor
+         */
         protected void mergeInfo(ModuleDescriptor parent) {
             ModuleRevisionId parentMrid = parent.getModuleRevisionId();
 
@@ -496,7 +508,7 @@ public class XmlModuleDescriptorParser extends AbstractModuleDescriptorParser {
                 mergeValue(parentMrid.getOrganisation(), currentMrid.getOrganisation()),
                 currentMrid.getName(),
                 mergeValue(parentMrid.getBranch(), currentMrid.getBranch()),
-                mergeValue(parentMrid.getRevision(), currentMrid.getRevision()),
+                mergeRevisionValue(parentMrid.getRevision(), currentMrid.getRevision()),
                 mergeValues(parentMrid.getQualifiedExtraAttributes(),
                             currentMrid.getQualifiedExtraAttributes())
             );
@@ -511,6 +523,14 @@ public class XmlModuleDescriptorParser extends AbstractModuleDescriptorParser {
             }
         }
         
+        private static String mergeRevisionValue(String inherited, String override) {
+            if (override==null || override.equals(Ivy.getWorkingRevision())) {
+                return inherited;
+            } else {
+                return override;
+            }
+        }
+        
         private static String mergeValue(String inherited, String override) {
             return override == null ? inherited : override;
         }
@@ -522,16 +542,32 @@ public class XmlModuleDescriptorParser extends AbstractModuleDescriptorParser {
             return dup;
         }
 
-        protected void mergeConfigurations(ModuleRevisionId sourceMrid, Configuration[] configurations) {
-            DefaultModuleDescriptor md = getMd();
+        /**
+         * Describes how to merge configurations elements
+         * @param sourceMrid the source module revision id
+         * @param configurations array of configurations to be inherited 
+         */
+        protected void mergeConfigurations(ModuleDescriptor parent) {
+            ModuleRevisionId sourceMrid = parent.getModuleRevisionId();
+            Configuration[] configurations = parent.getConfigurations();
             for (int i = 0; i < configurations.length; i++) {
                 Configuration configuration = configurations[i];
                 Message.debug("Merging configuration with: " + configuration.getName());
                 //copy configuration from parent descriptor
-                md.addConfiguration(new Configuration(configuration, sourceMrid));
+                getMd().addConfiguration(new Configuration(configuration, sourceMrid));
+            }
+
+            if (parent instanceof DefaultModuleDescriptor) {
+                setDefaultConfMapping(((DefaultModuleDescriptor) parent).getDefaultConfMapping());
+                setDefaultConf(((DefaultModuleDescriptor) parent).getDefaultConf());
+                getMd().setMappingOverride(((DefaultModuleDescriptor) parent).isMappingOverride());
             }
         }
 
+        /**
+         * Describes how dependencies should be inherited
+         * @param dependencies array of dependencies to inherit
+         */
         protected void mergeDependencies(DependencyDescriptor[] dependencies) {
             DefaultModuleDescriptor md = getMd();
             for (int i = 0; i < dependencies.length; i++) {
@@ -542,48 +578,69 @@ public class XmlModuleDescriptorParser extends AbstractModuleDescriptorParser {
             }
         }
 
+        /**
+         * Describes how to merge description
+         * @param description description going to be inherited
+         */
         protected void mergeDescription(String description) {
             String current = getMd().getDescription();
             if (current == null || current.trim().length() == 0) {
                 getMd().setDescription(description);
             }
         }
-
-        protected ModuleDescriptor parseOtherIvyFileOnFileSystem(String location)
-                throws ParseException, IOException {
-            URL url = null;
-            ModuleDescriptor parent = null;
-            url = getSettings().getRelativeUrlResolver().getURL(descriptorURL, location);
-            Message.debug("Trying to load included ivy file from " + url.toString());
-            URLResource res = new URLResource(url);
-            ModuleDescriptorParser parser = ModuleDescriptorParserRegistry.getInstance().getParser(
-                res);
-
-            parent = parser.parseDescriptor(getSettings(), url, isValidate());
-            return parent;
+        
+        /**
+         * Describes how to merge licenses
+         * @param licenses licenses going to be inherited
+         */
+        public void mergeLicenses(License[] licenses) {
+            for (int i = 0; i < licenses.length; i++) {
+                getMd().addLicense(licenses[i]);
+            }
         }
 
-        protected ModuleDescriptor parseOtherIvyFile(String parentOrganisation,
-                String parentModule, String parentRevision) throws ParseException {
-            ModuleId parentModuleId = new ModuleId(parentOrganisation, parentModule);
-            ModuleRevisionId parentMrid = new ModuleRevisionId(parentModuleId, parentRevision);
-
-            // try to load parent module in cache
-            File cacheFile = settings.getResolutionCacheManager().getResolvedIvyFileInCache(
-                ModuleRevisionId.newInstance(parentMrid, Ivy.getWorkingRevision()));
-            if (cacheFile.exists() && cacheFile.length() > 0) {
-                ModuleDescriptor md;
+        /**
+         * Returns the parent module using the location attribute (for dev purpose).
+         * @param location a given location 
+         * @throws IOException
+         * @throws ParseException 
+         */
+        private ModuleDescriptor parseParentModuleOnFilesystem(String location) throws IOException, ParseException {
+            if (!"file".equals(descriptorURL.getProtocol())) {
+                return null;
+            }
+            
+            File file = new File(location);
+            if (!file.isAbsolute()) {
+                URL url = settings.getRelativeUrlResolver().getURL(descriptorURL, location);
                 try {
-                    Message.debug("Trying to load included ivy file from cache");
-                    URL parentUrl = cacheFile.toURI().toURL();
-                    md = parseOtherIvyFileOnFileSystem(parentUrl.toString());
-                    return md;
-                } catch (IOException e) {
-                    // do nothing
-                    Message.error(e.getLocalizedMessage());
+                    file = new File(new URI(url.toExternalForm()));
+                } catch (URISyntaxException e) {
+                    file = new File(url.getPath());
                 }
             }
+            
+            file = FileUtil.normalize(file.getAbsolutePath());
+            if (!file.exists()) {
+                Message.verbose("Parent module doesn't exist on the filesystem: " + file.getAbsolutePath());
+                return null;
+            }
+            
+            FileResource res = new FileResource(null, file);
+            ModuleDescriptorParser parser = ModuleDescriptorParserRegistry.getInstance().getParser(res);
+            return parser.parseDescriptor(getSettings(), file.toURL(), res, isValidate());
+        }
 
+        /**
+         * Describe how to parse a {@link ModuleDescriptor} by asking repositories
+         * @param parentMrid a given {@link ModuleRevisionId} to find
+         * @return a {@link ModuleDescriptor} if found. Return null if no {@link ModuleDescriptor} was found
+         * @throws ParseException
+         */
+        protected ModuleDescriptor parseOtherIvyFile(ModuleRevisionId parentMrid) throws ParseException {
+            Message.debug(
+                "Trying to parse included ivy file by asking repository for module :"
+                            + parentMrid.toString());
             DependencyDescriptor dd = new DefaultDependencyDescriptor(parentMrid, true);
             ResolveData data = IvyContext.getContext().getResolveData();
             if (data == null) {
@@ -592,19 +649,13 @@ public class XmlModuleDescriptorParser extends AbstractModuleDescriptorParser {
                 options.setDownload(false);
                 data = new ResolveData(engine, options);
             }
-
             DependencyResolver resolver = getSettings().getResolver(parentMrid);
-            if (resolver == null) {
-                // TODO: Throw exception here?
-                return null;
-            } else {
-                dd = NameSpaceHelper.toSystem(dd, getSettings().getContextNamespace());
-                ResolvedModuleRevision otherModule = resolver.getDependency(dd, data);
-                if (otherModule == null) {
-                    throw new ParseException("Unable to find " + parentMrid.toString(), 0);
-                }
-                return otherModule.getDescriptor();
+            dd = NameSpaceHelper.toSystem(dd, getSettings().getContextNamespace());
+            ResolvedModuleRevision otherModule = resolver.getDependency(dd, data);
+            if (otherModule == null) {
+                throw new ParseException("Unable to find " + parentMrid.toString(), 0);
             }
+            return otherModule.getDescriptor();
 
         }
 
@@ -932,7 +983,7 @@ public class XmlModuleDescriptorParser extends AbstractModuleDescriptorParser {
             String pubDate = settings.substitute(attributes.getValue("publication"));
             if (pubDate != null && pubDate.length() > 0) {
                 try {
-                    getMd().setPublicationDate(Ivy.DATE_FORMAT.parse(pubDate));
+                    getMd().setPublicationDate(DateUtil.parse(pubDate));
                 } catch (ParseException e) {
                     addError("invalid publication date format: " + pubDate);
                     getMd().setPublicationDate(getDefaultPubDate());
@@ -1257,4 +1308,5 @@ public class XmlModuleDescriptorParser extends AbstractModuleDescriptorParser {
     public String toString() {
         return "ivy parser";
     }
+
 }
