@@ -436,7 +436,8 @@ public abstract class AbstractResolver
         return (CacheMetadataOptions) new CacheMetadataOptions()
             .setChangingMatcherName(getChangingMatcherName())
             .setChangingPattern(getChangingPattern())
-            .setCheckmodified(checkmodified)
+            .setCheckTTL(!data.getOptions().isUseCacheOnly())
+            .setCheckmodified(data.getOptions().isUseCacheOnly() ? Boolean.FALSE : checkmodified)
             .setValidate(doValidate(data))
             .setNamespace(getNamespace())
             .setForce(data.getOptions().isRefresh())
