@@ -108,7 +108,7 @@ public class IBiblioResolver extends URLResolver {
         }
     }
 
-    protected ResolvedResource findArtifactRef(Artifact artifact, Date date) {
+    public ResolvedResource findArtifactRef(Artifact artifact, Date date) {
         ensureConfigured(getSettings());
         ModuleRevisionId mrid = artifact.getModuleRevisionId();
         if (isM2compatible()) {
@@ -399,8 +399,17 @@ public class IBiblioResolver extends URLResolver {
                 List rres = new ArrayList();
                 for (Iterator iter = revs.iterator(); iter.hasNext();) {
                     String rev = (String) iter.next();
+                    ModuleRevisionId historicalMrid = ModuleRevisionId.newInstance(mrid, rev);
+
+                    String patternForRev = pattern;
+                    if (rev.endsWith("SNAPSHOT")) {
+                        String snapshotVersion = findSnapshotVersion(historicalMrid);
+                        if (snapshotVersion != null) {
+                            patternForRev = pattern.replaceFirst("\\-\\[revision\\]", "-" + snapshotVersion);
+                        }
+                    }
                     String resolvedPattern = IvyPatternHelper.substitute(
-                        pattern, ModuleRevisionId.newInstance(mrid, rev), artifact);
+                        patternForRev, historicalMrid, artifact);
                     try {
                         Resource res = repository.getResource(resolvedPattern);
                         if ((res != null) && res.exists()) {
