@@ -45,6 +45,7 @@ import org.apache.ivy.plugins.repository.Resource;
 import org.apache.ivy.plugins.resolver.util.ResolvedResource;
 import org.apache.ivy.plugins.resolver.util.ResolverHelper;
 import org.apache.ivy.plugins.resolver.util.ResourceMDParser;
+import org.apache.ivy.plugins.signer.SignatureGenerator;
 import org.apache.ivy.plugins.version.VersionMatcher;
 import org.apache.ivy.util.ChecksumHelper;
 import org.apache.ivy.util.FileUtil;
@@ -58,6 +59,8 @@ public class RepositoryResolver extends AbstractPatternsBasedResolver {
     private Repository repository;
 
     private Boolean alwaysCheckExactRevision = null;
+    
+    private String signerName = null;
 
     public RepositoryResolver() {
     }
@@ -76,6 +79,10 @@ public class RepositoryResolver extends AbstractPatternsBasedResolver {
             ((AbstractRepository) repository).setName(name);
         }
     }
+    
+    public void setSigner(String signerName) {
+        this.signerName = signerName;
+    }
 
     protected ResolvedResource findResourceUsingPattern(ModuleRevisionId mrid, String pattern,
             Artifact artifact, ResourceMDParser rmdparser, Date date) {
@@ -101,7 +108,7 @@ public class RepositoryResolver extends AbstractPatternsBasedResolver {
                                 ModuleDescriptorParserRegistry.getInstance().getParser(res);
                             ModuleDescriptor md = 
                                 parser.parseDescriptor(
-                                    getSettings(), temp.toURI().toURL(), res, false);
+                                    getParserSettings(), temp.toURI().toURL(), res, false);
                             revision = md.getRevision();
                             if ((revision == null) || (revision.length() == 0)) {
                                 revision = "working@" + name;
@@ -207,7 +214,7 @@ public class RepositoryResolver extends AbstractPatternsBasedResolver {
         String dest = getDestination(destPattern, artifact, mrid);
 
         put(artifact, src, dest, overwrite);
-        Message.info("\tpublished " + artifact.getName() + " to " + hidePassword(dest));
+        Message.info("\tpublished " + artifact.getName() + " to " + hidePassword(repository.standardize(dest)));
     }
 
     protected String getDestination(String pattern, Artifact artifact, ModuleRevisionId mrid) {
@@ -223,11 +230,15 @@ public class RepositoryResolver extends AbstractPatternsBasedResolver {
                 throw new IllegalArgumentException("Unknown checksum algorithm: " + checksums[i]);
             }
         }
-        
+
         repository.put(artifact, src, dest, overwrite);
         for (int i = 0; i < checksums.length; i++) {
             putChecksum(artifact, src, dest, overwrite, checksums[i]);
         }
+
+        if (signerName != null) {
+            putSignature(artifact, src, dest, overwrite);
+        }
     }
 
     protected void putChecksum(Artifact artifact, File src, String dest, boolean overwrite,
@@ -243,6 +254,25 @@ public class RepositoryResolver extends AbstractPatternsBasedResolver {
         }
     }
 
+    protected void putSignature(Artifact artifact, File src, String dest, boolean overwrite) throws IOException {
+        SignatureGenerator gen = getSettings().getSignatureGenerator(signerName);
+        if (gen == null) {
+            throw new IllegalArgumentException("Couldn't sign the artifacts! " +
+                    "Unknown signer name: '" + signerName + "'");
+        }
+
+        File tempFile = File.createTempFile("ivytemp", gen.getExtension());
+
+        try {
+            gen.sign(src, tempFile);
+            repository.put(DefaultArtifact.cloneWithAnotherTypeAndExt(artifact, 
+                    gen.getExtension(), artifact.getExt() + "." + gen.getExtension()), 
+                    tempFile, dest + "." + gen.getExtension(), overwrite);
+        } finally {
+            tempFile.delete();
+        }
+    }
+
     public DownloadReport download(Artifact[] artifacts, DownloadOptions options) {
         EventManager eventManager = getEventManager();
         try {
