@@ -21,6 +21,7 @@ import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.OutputStream;
 import java.net.URL;
 import java.net.UnknownHostException;
 import java.text.ParseException;
@@ -44,8 +45,8 @@ import org.apache.commons.httpclient.auth.CredentialsNotAvailableException;
 import org.apache.commons.httpclient.auth.CredentialsProvider;
 import org.apache.commons.httpclient.methods.GetMethod;
 import org.apache.commons.httpclient.methods.HeadMethod;
-import org.apache.commons.httpclient.methods.InputStreamRequestEntity;
 import org.apache.commons.httpclient.methods.PutMethod;
+import org.apache.commons.httpclient.methods.RequestEntity;
 import org.apache.commons.httpclient.params.HttpMethodParams;
 import org.apache.ivy.Ivy;
 import org.apache.ivy.util.CopyProgressListener;
@@ -126,24 +127,15 @@ public class HttpClientHandler extends AbstractURLHandler {
     }
 
     public void upload(File src, URL dest, CopyProgressListener l) throws IOException {
-        HttpClient client = getClient(dest);
+        HttpClient client = getClient();
 
         PutMethod put = new PutMethod(normalizeToString(dest));
         put.setDoAuthentication(useAuthentication(dest) || useProxyAuthentication());
-        FileInputStream fileStream = null;
         try {
-            fileStream = new FileInputStream(src);
-            put.setRequestEntity(new InputStreamRequestEntity(fileStream));
+            put.setRequestEntity(new FileRequestEntity(src));
             int statusCode = client.executeMethod(put);
             validatePutStatusCode(dest, statusCode, null);
         } finally {
-            if (fileStream != null) {
-                try {
-                    fileStream.close();
-                } catch (IOException e) {
-                    /* ignored */
-                }
-            }
             put.releaseConnection();
         }
     }
@@ -161,7 +153,9 @@ public class HttpClientHandler extends AbstractURLHandler {
                 method = doGet(url, timeout);
             }
             if (checkStatusCode(url, method)) {
-                return new URLInfo(true, getResponseContentLength(method), getLastModified(method));
+                String contentType = method.getResponseHeader("content-type").getValue();
+                String bodyCharset = BasicURLHandler.getCharSetFromContentType(contentType);
+                return new URLInfo(true, getResponseContentLength(method), getLastModified(method), bodyCharset);
             }
         } catch (HttpException e) {
             Message.error("HttpClientHandler: " + e.getMessage() + ":" + e.getReasonCode() + "="
@@ -188,6 +182,12 @@ public class HttpClientHandler extends AbstractURLHandler {
         if (status == HttpStatus.SC_OK) {
             return true;
         }
+        
+        // IVY-1328: some servers return a 204 on a HEAD request
+        if ("HEAD".equals(method.getName()) && (status == 204)) {
+            return true;
+        }
+        
         Message.debug("HTTP response status: " + status + " url=" + url);
         if (status == HttpStatus.SC_PROXY_AUTHENTICATION_REQUIRED) {
             Message.warn("Your proxy requires authentication.");
@@ -247,7 +247,7 @@ public class HttpClientHandler extends AbstractURLHandler {
     }
 
     private GetMethod doGet(URL url, int timeout) throws IOException {
-        HttpClient client = getClient(url);
+        HttpClient client = getClient();
         client.setTimeout(timeout);
 
         GetMethod get = new GetMethod(normalizeToString(url));
@@ -258,7 +258,7 @@ public class HttpClientHandler extends AbstractURLHandler {
     }
 
     private HeadMethod doHead(URL url, int timeout) throws IOException {
-        HttpClient client = getClient(url);
+        HttpClient client = getClient();
         client.setTimeout(timeout);
 
         HeadMethod head = new HeadMethod(normalizeToString(url));
@@ -267,7 +267,7 @@ public class HttpClientHandler extends AbstractURLHandler {
         return head;
     }
 
-    private HttpClient getClient(URL url) {
+    private HttpClient getClient() {
         if (httpClient == null) {
             final MultiThreadedHttpConnectionManager connManager = 
                 new MultiThreadedHttpConnectionManager();
@@ -399,4 +399,34 @@ public class HttpClientHandler extends AbstractURLHandler {
         
         return new NTCredentials(user, password, HostUtil.getLocalHostName(), domain);
     }
+    
+    private static class FileRequestEntity implements RequestEntity {
+        private File file;
+        
+        public FileRequestEntity(File file) {
+            this.file = file;
+        }
+
+        public long getContentLength() {
+            return file.length();
+        }
+
+        public String getContentType() {
+            return null;
+        }
+
+        public boolean isRepeatable() {
+            return true;
+        }
+
+        public void writeRequest(OutputStream out) throws IOException {
+            InputStream instream = new FileInputStream(file);
+            try {
+                FileUtil.copy(instream, out, null, false);
+            } finally {
+                instream.close();
+            }
+        }            
+    }
+
 }
