@@ -18,6 +18,7 @@
 package org.apache.ivy.util.url;
 
 import java.lang.reflect.Field;
+import java.lang.reflect.Method;
 import java.net.Authenticator;
 import java.net.PasswordAuthentication;
 
@@ -79,8 +80,7 @@ public final class IvyAuthenticator extends Authenticator {
     protected PasswordAuthentication getPasswordAuthentication() {
         PasswordAuthentication result = null;
         
-        String proxyHost = System.getProperty("http.proxyHost");
-        if (getRequestingHost().equals(proxyHost)) {
+        if (isProxyAuthentication()) {
             String proxyUser = System.getProperty("http.proxyUser");
             if ((proxyUser != null) && (proxyUser.trim().length() > 0)) {
                 String proxyPass = System.getProperty("http.proxyPassword", "");
@@ -94,7 +94,8 @@ public final class IvyAuthenticator extends Authenticator {
                     + Credentials.buildKey(getRequestingPrompt(), getRequestingHost()) + "' c='" + c
                     + "'");
             if (c != null) {
-                result = new PasswordAuthentication(c.getUserName(), c.getPasswd().toCharArray());
+                final String password = c.getPasswd() == null ? "" : c.getPasswd();
+                result = new PasswordAuthentication(c.getUserName(), password.toCharArray());
             }
         }
         
@@ -111,5 +112,46 @@ public final class IvyAuthenticator extends Authenticator {
         
         return result;
     }
+    
+    /**
+     * Checks if the current authentication request is for the proxy server.
+     * This functionality is not available in JDK1.4, so we check this in a
+     * very dirty way which is probably not very portable, but will work for
+     * the SUN 1.4 JDKs.
+     * 
+     * @return
+     */
+    private boolean isProxyAuthentication() {
+        try {
+            // we first try to invoke the getRequestorType() method which is a JDK1.5+ method
+            Method m = Authenticator.class.getDeclaredMethod("getRequestorType", null);
+            Object result = m.invoke(this, null);
+            return "PROXY".equals(String.valueOf(result));
+        } catch (NoSuchMethodException e) {
+            // do nothing, this is a JDK1.5+ method
+        } catch (Throwable t) {
+            Message.debug("Error occurred while checking if the authentication request is for the proxy server: " + t.getMessage());            
+        }
+        
+        // now we will do something very dirty and analyse the stack trace to see
+        // if this method is called from within the 'getHttpProxyAuthentication' method
+        // or the 'getServerAuthentication' method which are both part of the 
+        // sun.net.www.protocol.http.HttpURLConnection class.
+        // This might not work on other 1.4 JVM's!
+        // This code should be removed when Ivy requires JDK1.5+
+        StackTraceElement[] stackTrace = (new Exception()).getStackTrace();
+        for (int i = 0; i < stackTrace.length; i++) {
+            if ("getHttpProxyAuthentication".equals(stackTrace[i].getMethodName())) {
+                return true;
+            }
+            if ("getServerAuthentication".equals(stackTrace[i].getMethodName())) {
+                return false;
+            }
+        }
+        
+        // fallback to the Ivy 2.2.0 behavior
+        String proxyHost = System.getProperty("http.proxyHost");
+        return getRequestingHost().equals(proxyHost);
+    }
 
 }
